"""
MIT License

Copyright (c) 2019 thecosmos
Copyright (c) 2024 hunter87ff

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE."""

from .base import DiscordModelsBase
from flask import current_app
from .channel import Channel
from ..role import Role
from .member import Member
from ..emoji import Emoji
from .. import types
from .. import configs
import json
from typing import (
    Any,
    List,
    Dict,
    Sequence
)

class Guild(DiscordModelsBase):
    MANY = True
    ROUTE = "/guilds/{guild_id}"

    def __init__(self, payload:dict):
        super().__init__(payload)
        self.id:int = int(self._payload["id"])
        self.name:str = self._payload["name"]
        self.icon_hash:str = self._payload.get("icon")
        self.is_owner = self._payload.get("owner")
        self.permissions = self.__get_permissions(self._payload.get("permissions"))
        self._roles:dict= self._add_roles()
        self._members:dict = {}
        self._channels:dict = self._add_channels()
        self._owner_id:int = int(self._payload.get("owner_id"))

    def _add_channels(self):
        fetched_channels = self._bot_request(f"/guilds/{self.id}/channels")
        return {channel['id']:Channel(channel, self) for channel in fetched_channels}
    
    def _add_members(self):
        fetched_members = self._bot_request(f"/guilds/{self.id}/members?limit=1000")
        self._members = {member["user"]["id"]: Member(member, self) for member in fetched_members}

    def _add_roles(self):
        fetched_roles = self._payload.get("roles")
        return {role["id"] : Role(role, self) for role in fetched_roles}

    @staticmethod
    def __get_permissions(permissions_value):
        if permissions_value is None:return
        return types.Permissions(int(permissions_value))

    def __str__(self):
        return self.name

    def __eq__(self, guild):
        return isinstance(guild, Guild) and guild.id == self.id

    def __ne__(self, guild):
        return not self.__eq__(guild)

    @property
    def icon_url(self)->str:
        if not self.icon_hash:return
        return configs.DISCORD_GUILD_ICON_BASE_URL.format(guild_id=self.id, icon_hash=self.icon_hash)
    
    def get_channel(self, channel_id:int)->Channel:
        return self._channels.get(channel_id)
    
    def get_member(self, user_id:int)->Member:
        if self._members and user_id in self._members:return self._members[user_id]
        fetched_member = self._bot_request(f"/guilds/{self.id}/members/{user_id}")
        member_obj = Member(fetched_member, self)
        if not self._members: self._members[member_obj.id] = member_obj
        return member_obj
    
    def get_role(self, role_id:int)->Role:
        return self._roles.get(role_id)    
    
    @property
    def channels(self) -> List[Channel]:
        return self._channels.values()
    
    @property
    def emojies(self)->List[str]:
        return [Emoji(emoji) for emoji in self._payload.get("emojis", [])]
    
    @property
    def features(self)->List[str]:
        return self._payload.get("features", [])
    
    @property
    def members(self)->List[Member]:
        return self._members.values()
    
    @property
    def roles(self)->List[Role]:
        return self._roles.values() 

    @property
    def owner_id(self)->int:
        return self._owner_id

    

class CGuild(DiscordModelsBase):
    """CGuild Class representing discord Guild the user is part of.

    Operations
    ----------
    x == y
        Checks if two guild's are the same.
    x != y
        Checks if two guild's are not the same.
    str(x)
        Returns the guild's name.

    Attributes
    ----------
    id : int
        Discord ID of the guild.
    name : str
        Name of the guild.
    icon_hash : str
        Hash of guild's icon.
    is_owner : bool
        Boolean determining if current user is owner of the guild or not.
    permissions : discord.Permissions
        An instance of discord.Permissions representing permissions of current user in the guild.

    """

    MANY = True
    ROUTE = "/users/@me/guilds"

    def __init__(self, payload):
        super().__init__(payload)
        self.id = int(self._payload["id"])
        self.name = self._payload["name"]
        self.icon_hash = self._payload.get("icon")
        self.is_owner = self._payload.get("owner")
        self.permissions = self.__get_permissions(self._payload.get("permissions"))

    @staticmethod
    def __get_permissions(permissions_value):
        if permissions_value is None:
            return
        return types.Permissions(int(permissions_value))

    def __str__(self):
        return self.name

    def __eq__(self, guild):
        return isinstance(guild, Guild) and guild.id == self.id

    def __ne__(self, guild):
        return not self.__eq__(guild)

    @property
    def icon_url(self):
        """A property returning direct URL to the guild's icon. Returns None if guild has no icon set."""
        if not self.icon_hash:
            return
        return configs.DISCORD_GUILD_ICON_BASE_URL.format(guild_id=self.id, icon_hash=self.icon_hash)

    @classmethod
    def fetch_from_api(cls, cache=True):
        """A class method which returns an instance or list of instances of this model by implicitly making an
        API call to Discord. If an instance of :py:class:`discord_flask.User` exists in the users internal cache
        who belongs to these guilds then, the cached property :py:attr:`discord_flask.User.guilds` is updated.

        Parameters
        ----------
        cache : bool
            Determines if the :py:attr:`discord_flask.User.guilds` cache should be updated with the new guilds.

        Returns
        -------
        list[discord_flask.Guild, ...]
            List of instances of :py:class:`discord_flask.Guild` to which this user belongs.

        """
        guilds = super().fetch_from_api()

        if cache:
            user = current_app.discord.users_cache.get(current_app.discord.user_id)
            try:
                user.guilds = {guild.id: guild for guild in guilds}
            except AttributeError:
                pass

        return guilds
    
    def get_member(self, user_id:int)->Member:
        try:
            fetched_member = self._bot_request(f"/guilds/{self.id}/members/{user_id}")
            print(fetched_member)
            return Member(fetched_member, self)
        except:
            return None


from .. import types



class Role():
    MANY = True
    ROUTE = "/guilds/{guild_id}/roles"
    # print(json.dumps(_payload, indent=4))
    def __init__(self, _payload, guild):
        self.id = int(_payload["id"])
        self.name = _payload["name"]
        self.permissions = self.__get_permissions(_payload.get("permissions"))
        self.mention:str = f"<@{self.id}>"
        # self.last_message_id = payload["last_message_id"]
        self.guild = guild

        
    @staticmethod
    def __get_permissions(permissions_value):
        if permissions_value is None:
            return
        return types.Permissions(int(permissions_value))


import jwt
import typing
from cachetools import cached, LFUCache
from . import configs, _http, models, utils, exceptions, types
from flask import request, session, redirect, current_app
from oauthlib.common import add_params_to_uri, generate_token


class Session(_http.DiscordOAuth2HttpClient):
    """Main client class representing hypothetical OAuth2 session with discord.
    It uses Flask `session <http://flask.pocoo.org/docs/1.0/api/#flask.session>`_ local proxy object
    to save state, authorization token and keeps record of users sessions across different requests.
    This class inherits :py:class:`discord_flask._http.DiscordOAuth2HttpClient` class.

    Parameters
    ----------
    app : Flask
        An instance of your `flask application <http://flask.pocoo.org/docs/1.0/api/#flask.Flask>`_.
    client_id : int, optional
        The client ID of discord application provided. Can be also set to flask config
        with key ``DISCORD_CLIENT_ID``.
    client_secret : str, optional
        The client secret of discord application provided. Can be also set to flask config
        with key ``DISCORD_CLIENT_SECRET``.
    redirect_uri : str, optional
        The default URL to use to redirect user to after authorization. Can be also set to flask config
        with key ``DISCORD_REDIRECT_URI``.
    bot_token : str, optional
        The bot token of the application. This is required when you also need to access bot scope resources
        beyond the normal resources provided by the OAuth. Can be also set to flask config with
        key ``DISCORD_BOT_TOKEN``.
    users_cache : cachetools.LFUCache, optional
        Any dict like mapping to internally cache the authorized users. Preferably an instance of
        cachetools.LFUCache or cachetools.TTLCache. If not specified, default cachetools.LFUCache is used.
        Uses the default max limit for cache if ``DISCORD_USERS_CACHE_MAX_LIMIT`` isn't specified in app config.

    Attributes
    ----------
    client_id : int
        The client ID of discord application provided.
    redirect_uri : str
        The default URL to use to redirect user to after authorization.
    users_cache : cachetools.LFUCache
        A dict like mapping to internally cache the authorized users. Preferably an instance of
        cachetools.LFUCache or cachetools.TTLCache. If not specified, default cachetools.LFUCache is used.
        Uses the default max limit for cache if ``DISCORD_USERS_CACHE_MAX_LIMIT`` isn't specified in app config.

    """

    @staticmethod
    def __save_state(state):
        session["DISCORD_OAUTH2_STATE"] = state

    @staticmethod
    def __get_state():
        return session.get("DISCORD_OAUTH2_STATE", str())

    def create_session(
            self, scope: list = None, *, data: dict = None, prompt: bool = True,
            permissions: typing.Union[types.Permissions, int] = 0, **params
    ):
        """Primary method used to create OAuth2 session and redirect users for
        authorization code grant.

        Parameters
        ----------
        scope : list, optional
            An optional list of valid `Discord OAuth2 Scopes
            <https://discordapp.com/developers/docs/topics/oauth2#shared-resources-oauth2-scopes>`_.
        data : dict, optional
            A mapping of your any custom data which you want to access after authorization grant. Use
            `:py:meth:discord_flask.Session.callback` to retrieve this data in your callback view.
        prompt : bool, optional
            Determines if the OAuth2 grant should be explicitly prompted and re-approved. Defaults to True.
            Specify False for implicit grant which will skip the authorization screen and redirect to redirect URI.
        permissions: typing.Union[discord_flask.types.Permissions, int], optional
            An optional parameter determining guild permissions of the bot while adding it to a guild using
            discord OAuth2 with `bot` scope. It is same as generating so called *bot invite link* which redirects
            to your callback endpoint after bot authorization flow. Defaults to 0 or no permissions.
        params : kwargs, optional
            Additional query parameters to append to authorization URL for customized OAuth flow.

        Returns
        -------
        redirect
            Flask redirect to discord authorization servers to complete authorization code grant process.

        """
        scope = scope or request.args.get("scope", str()).split() or configs.DISCORD_OAUTH_DEFAULT_SCOPES

        if not prompt and set(scope) & set(configs.DISCORD_PASSTHROUGH_SCOPES):
            raise ValueError("You should use explicit OAuth grant for passthrough scopes like bot.")

        data = data or dict()
        data["__state_secret_"] = generate_token()

        state = jwt.encode(data, current_app.config["SECRET_KEY"], algorithm="HS256")

        discord_session = self._make_session(scope=scope, state=state)
        authorization_url, state = discord_session.authorization_url(configs.DISCORD_AUTHORIZATION_BASE_URL)

        self.__save_state(state)

        params = params or dict()
        params["prompt"] = "consent" if prompt else "none"
        if "bot" in scope:
            if not isinstance(permissions, (types.Permissions, int)):
                raise ValueError(f"Passed permissions must be an int or discord.Permissions, not {type(permissions)}.")
            if isinstance(permissions, types.Permissions):
                permissions = permissions.value
            params["permissions"] = permissions
            try:
                params["disable_guild_select"] = utils.json_bool(params["disable_guild_select"])
            except KeyError:
                pass
        authorization_url = add_params_to_uri(authorization_url, params)

        return redirect(authorization_url)

    @staticmethod
    def save_authorization_token(token: dict):
        """A staticmethod which saves a dict containing Discord OAuth2 token and other secrets to the user's cookies.
        Meaning by default, it uses client side session handling.

        Override this method if you want to handle the user's session server side. If this method is overridden then,
        you must also override :py:meth:`discord_flask.Session.get_authorization_token`.

        """
        session["DISCORD_OAUTH2_TOKEN"] = token

    @staticmethod
    def get_authorization_token() -> dict:
        """A static method which returns a dict containing Discord OAuth2 token and other secrets which was saved
        previously by `:py:meth:`discord_flask.Session.save_authorization_token` from user's cookies.

        You must override this method if you are implementing server side session handling.

        """
        return session.get("DISCORD_OAUTH2_TOKEN")

    def callback(self):
        """A method which should be always called after completing authorization code grant process
        usually in callback view.
        It fetches the authorization token and saves it flask
        `session <http://flask.pocoo.org/docs/1.0/api/#flask.session>`_ object.

        """
        error = request.values.get("error")
        if error:
            if error == "access_denied":
                raise exceptions.AccessDenied()
            raise exceptions.HttpException(error)

        state = self.__get_state()
        token = self._fetch_token(state)
        self.save_authorization_token(token)

        return jwt.decode(state, current_app.config["SECRET_KEY"], algorithms="HS256")

    def revoke(self):
        """This method clears current discord token, state and all session data from flask
        `session <http://flask.pocoo.org/docs/1.0/api/#flask.session>`_. Which means user will have
        to go through discord authorization token grant flow again. Also tries to remove the user from internal
        cache if they exist.

        """

        self.users_cache.pop(self.user_id, None)

        for session_key in self.SESSION_KEYS:
            try:
                session.pop(session_key)
            except KeyError:
                pass

    @property
    def authorized(self):
        """A boolean indicating whether current session has authorization token or not."""
        return self._make_session().authorized

    @staticmethod
    def fetch_user() -> models.User:
        """This method returns user object from the internal cache if it exists otherwise makes an API call to do so.

        Returns
        -------
        discord_flask.models.User

        """
        return models.User.get_from_cache() or models.User.fetch_from_api()

    @staticmethod
    def fetch_connections() -> list[models.UserConnection]:
        """This method returns list of user connection objects from internal cache if it exists otherwise
        makes an API call to do so.

        Returns
        -------
        list
            List of :py:class:`discord_flask.models.UserConnection` objects.

        """
        user:models.User = models.User.get_from_cache()
        try:
            if user.connections is not None:
                return user.connections
        except AttributeError:
            pass

        return models.UserConnection.fetch_from_api()
    
    @cached(cache=LFUCache(maxsize=1024))
    def get_guild(self, guild_id:int) -> models.Guild:
        ROUTE = f"/guilds/{guild_id}"
        if self._guilds.get(guild_id):
            return self._guilds.get(guild_id)
        payload = current_app.discord.bot_request(ROUTE)
        guild = models.Guild(payload)
        self._guilds[guild_id] = guild
        return guild



    @staticmethod
    def fetch_guilds() -> list[models.CGuild]:
        user:models.User = models.User.get_from_cache()
        try:
            if user.guilds is not None:
                return user.guilds
        except AttributeError:
            pass

        return models.CGuild.fetch_from_api()
        



# This code is mostly copied from flask_discord
# Author: https://github.com/weibeu

import cachetools
import requests
import typing
import json
import abc
from .message import Message
from .embed import Embed
from . import configs
from . import exceptions
from .models import Guild
from flask import Flask, session, request
from collections.abc import Mapping
from requests_oauthlib import OAuth2Session


class DiscordOAuth2HttpClient(abc.ABC):
    """An OAuth2 http abstract base class providing some factory methods.
    This class is meant to be overridden by :py:class:`discord_flask.Session` and should not be
    used directly.

    """

    SESSION_KEYS = [
        "DISCORD_USER_ID",
        "DISCORD_OAUTH2_STATE",
        "DISCORD_OAUTH2_TOKEN",
    ]

    def __init__(
            self, app:Flask=None,
            client_id=None, client_secret=None, redirect_uri=None,
            bot_token=None, users_cache=None, proxy=None, proxy_auth=None
    ):
        self.client_id = client_id
        self.__client_secret = client_secret
        self.redirect_uri = redirect_uri
        self.__bot_token = bot_token
        self.users_cache = users_cache
        self.proxy = proxy
        self.proxy_auth = proxy_auth
        self._guilds: dict[int:Guild] = {}

        if app is not None:
            self.init_app(app)

    def init_app(self, app:Flask):
        """A method to lazily initialize the application.
        Use this when you're using flask factory pattern to create your instances of your flask application.

        Parameters
        ----------
        app : Flask
            An instance of your `flask application <http://flask.pocoo.org/docs/1.0/api/#flask.Flask>`_.

        """
        self.client_id = self.client_id or app.config["DISCORD_CLIENT_ID"]
        self.__client_secret = self.__client_secret or app.config["DISCORD_CLIENT_SECRET"]
        self.redirect_uri = self.redirect_uri or app.config["DISCORD_REDIRECT_URI"]
        self.__bot_token = self.__bot_token or app.config.get("DISCORD_BOT_TOKEN", str())
        self.users_cache = cachetools.LFUCache(
            app.config.get("DISCORD_USERS_CACHE_MAX_LIMIT", configs.DISCORD_USERS_CACHE_DEFAULT_MAX_LIMIT)
        ) if self.users_cache is None else self.users_cache
        if not issubclass(self.users_cache.__class__, Mapping):
            raise ValueError("Instance users_cache must be a mapping like object.")
        self.proxy = self.proxy or app.config.get("DISCORD_PROXY_SETTINGS")
        self.proxy_auth = self.proxy_auth or app.config.get("DISCORD_PROXY_AUTH_SETTINGS")
        app.discord = self

    @property
    def user_id(self) -> typing.Union[int, None]:
        """A property which returns Discord user ID if it exists in flask :py:attr:`flask.session` object.

        Returns
        -------
        int
            The Discord user ID of current user.
        None
            If the user ID doesn't exists in flask :py:attr:`flask.session`.

        """
        return session.get("DISCORD_USER_ID")

    @staticmethod
    @abc.abstractmethod
    def save_authorization_token(token: dict):
        raise NotImplementedError

    @staticmethod
    @abc.abstractmethod
    def get_authorization_token() -> dict:
        raise NotImplementedError

    def _fetch_token(self, state):
        discord = self._make_session(state=state)
        return discord.fetch_token(
            configs.DISCORD_TOKEN_URL,
            client_secret=self.__client_secret,
            authorization_response=request.url
        )

    def _make_session(self, token: str = None, state: str = None, scope: list = None) -> OAuth2Session:
        """A low level method used for creating OAuth2 session.

        Parameters
        ----------
        token : str, optional
            The authorization token to use which was previously received from authorization code grant.
        state : str, optional
            The state to use for OAuth2 session.
        scope : list, optional
            List of valid `Discord OAuth2 Scopes
            <https://discordapp.com/developers/docs/topics/oauth2#shared-resources-oauth2-scopes>`_.

        Returns
        -------
        OAuth2Session
            An instance of OAuth2Session class.

        """
        return OAuth2Session(
            client_id=self.client_id,
            token=token or self.get_authorization_token(),
            state=state,
            scope=scope,
            redirect_uri=self.redirect_uri,
            auto_refresh_kwargs={
                'client_id': self.client_id,
                'client_secret': self.__client_secret,
            },
            auto_refresh_url=configs.DISCORD_TOKEN_URL,
            token_updater=self.save_authorization_token)

    def request(self, route: str, method="GET", data=None, oauth=True, **kwargs) -> typing.Union[dict, str]:
        """Sends HTTP request to provided route or discord endpoint.

        Note
        ----
        It automatically prefixes the API Base URL so you will just have to pass routes or URL endpoints.

        Parameters
        ----------
        route : str
            Route or endpoint URL to send HTTP request to. Example: ``/users/@me``
        method : str, optional
            Specify the HTTP method to use to perform this request.
        data : dict, optional
            The optional payload the include with the request.
        oauth : bool
            A boolean determining if this should be Discord OAuth2 session request or any standard request.

        Returns
        -------
        dict, str
            Dictionary containing received from sent HTTP GET request if content-type is ``application/json``
            otherwise returns raw text content of the response.

        Raises
        ------
        discord_flask.Unauthorized
            Raises :py:class:`discord_flask.Unauthorized` if current user is not authorized.
        discord_flask.RateLimited
            Raises an instance of :py:class:`discord_flask.RateLimited` if application is being rate limited by Discord.

        """
        route = configs.DISCORD_API_BASE_URL + route

        if self.proxy is not None:
            kwargs["proxy"] = self.proxy
        if self.proxy_auth is not None:
            kwargs["proxy_auth"] = self.proxy_auth

        response = self._make_session(
        ).request(method, route, data, **kwargs) if oauth else requests.request(method, route, data=data, **kwargs)

        if response.status_code == 401:
            raise exceptions.Unauthorized()
        if response.status_code == 429:
            raise exceptions.RateLimited(response.json(), response.headers)

        try:
            return response.json()
        except json.JSONDecodeError:
            return response.text

    def bot_request(self, route: str, method="GET", **kwargs) -> typing.Union[dict, str]:
        """Make HTTP request to specified endpoint with bot token as authorization headers.

        Parameters
        ----------
        route : str
            Route or endpoint URL to send HTTP request to.
        method : str, optional
            Specify the HTTP method to use to perform this request.

        Returns
        -------
        dict, str
            Dictionary containing received from sent HTTP GET request if content-type is ``application/json``
            otherwise returns raw text content of the response.

        Raises
        ------
        discord_flask.Unauthorized
            Raises :py:class:`discord_flask.Unauthorized` if current user is not authorized.
        discord_flask.RateLimited
            Raises an instance of :py:class:`discord_flask.RateLimited` if application is being rate limited by Discord.

        """
        headers = {"Authorization": f"Bot {self.__bot_token}"}
        return self.request(route, method=method, oauth=False, headers=headers, **kwargs)
    

    def send_message(self, channel_id: int, content:str=None, embeds:list[Embed]=None) -> Message:
        """Send a message to specified channel."""
        data = {
            "content": content,
            "embeds": [embed.to_dict() for embed in embeds if embeds]
        }
        r = self.bot_request(method='POST', route=f'/channels/{channel_id}/messages', json=data)
        return Message(r)




from flask import current_app

from .base import DiscordModelsBase
from .integration import Integration


class UserConnection(DiscordModelsBase):
    """Class representing connections in discord account of the user.

    Attributes
    ----------
    id : str
        ID of the connection account.
    name : str
        The username of the connection account.
    type : str
        The service of connection (twitch, youtube).
    revoked : bool
        A boolean representing whether the connection is revoked.
    integrations : list
        A list of server Integration objects.
    verified : bool
        A boolean representing whether the connection is verified.
    friend_sync : bool
        A boolean representing whether friend sync is enabled for this connection.
    show_activity : bool
        A boolean representing whether activities related to this connection will
        be shown in presence updates.
    visibility : int
        An integer representing
        `visibility <https://discordapp.com/developers/docs/resources/user#user-object-visibility-types>`_
        of this connection.

    """

    MANY = True
    ROUTE = "/users/@me/connections"

    def __init__(self, payload):
        super().__init__(payload)
        self.id = self._payload["id"]
        self.name:str = self._payload.get("name")
        self.type = self._payload.get("type")
        self.revoked = self._payload.get("revoked")
        self.integrations = self.__get_integrations()
        self.verified = self._payload.get("verified")
        self.friend_sync = self._payload.get("friend_sync")
        self.show_activity = self._payload.get("show_activity")
        self.visibility = self._payload.get("visibility")

    def __get_integrations(self):
        return [Integration(payload) for payload in self._payload.get("integrations", list())]

    @property
    def is_visible(self):
        """A property returning bool if this integration is visible to everyone."""
        return bool(self.visibility)

    @classmethod
    def fetch_from_api(cls, cache=True):
        """A class method which returns an instance or list of instances of this model by implicitly making an
        API call to Discord. If an instance of :py:class:`discord_flask.User` exists in the users internal cache
        who are attached to these connections then, the cached property :py:attr:`discord_flask.User.connections`
        is updated.

        Parameters
        ----------
        cache : bool
            Determines if the :py:attr:`discord_flask.User.guilds` cache should be updated with the new guilds.

        Returns
        -------
        list[discord_flask.UserConnection, ...]
            List of instances of :py:class:`discord_flask.UserConnection` to which this user belongs.

        """
        connections = super().fetch_from_api()

        if cache:
            user = current_app.discord.users_cache.get(current_app.discord.user_id)
            try:
                user.connections = connections
            except AttributeError:
                pass

        return connections



"""MIT License

Copyright (c) 2024 hunter87ff

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE."""



from flask import current_app
from .. import types
from ..embed import Embed
from ..message import Message

class Channel():
    MANY = True
    ROUTE = "/channels/{channel_id}"

    def __init__(self, payload, guild):
        self._payload:dict = payload
        self.id:int = int(self._payload["id"])
        self.name:str = self._payload["name"]
        self.permissions = self.__get_permissions(self._payload.get("permissions"))
        self.mention:str = f"<#{self.id}>"
        # self.last_message_id = self._payload["last_message_id"]
        self._guild = guild

    @property
    def guild(self):
       return self._guild

    @staticmethod
    def __get_permissions(permissions_value) -> types.Permissions:
        if permissions_value is None:return
        return types.Permissions(int(permissions_value))
    
    def send(self, content:str=None, embed:Embed = None) -> Message:
        """Send a message to the channel.

        Parameters
        ----------
        content : str
            Content of the message.
        embed : discord.Embed, optional
            Embed to be sent with the message.

        Returns
        -------
        discord.Message
            Message object of the sent message.
        """
        return current_app.discord.send_message(channel_id=self.id, content=content, embeds=[embed])  
    




"""
Discord-Flask
-------------

A feature rich discord extension for Flask.
"""
import re
from setuptools import setup, find_packages

def __get_version():
    with open("discord_flask/__init__.py") as package_init_file:
        return re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', package_init_file.read(), re.MULTILINE).group(1)

requirements = [
    "Flask",
    "pyjwt>=2.4.0",
    "requests",
    "oauthlib",
    "cachetools",
    "requests_oauthlib",
    "typing-extensions"
]
def __get_long_description():
    with open("README.rst", "r") as readme_file:
        return readme_file.read()

setup(
    name='Discord-Flask',
    version=__get_version(),
    url='https://github.com/hunter87ff/Discord-Flask',
    license='MIT',
    author='hunter87ff',
    author_email='hunter87.dev@gmail.com',
    long_description=__get_long_description(),
    packages=find_packages(),
    zip_safe=False,
    include_package_data=True,
    platforms='any',
    install_requires=requirements,
    classifiers=[
        'Framework :: Flask',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)



from . import exceptions
from .utils import requires_authorization 
from .client import Session
from .models import *
from .embed import Embed
from .emoji import Emoji
from .message import Message
from .role import Role

__all__ = [
    "Session",
    "exceptions",
    "Embed",
    "Emoji",
    "Message",
    "Role",
    "Guild",
    "User",
    "Channel",
    "Permissions",
    "Member",
]

__author__ = "hunter87ff"
__version__ = "1.0.4"
